//2013.01.15 Introduction to SuperCollider
//Written by Eli Fieldsteel <eli.fieldsteel@gmail.com>

INTRODUCTION

SuperCollider (SC) is a free, open source, cross-platform, object-oriented programming environment for real-time audio synthesis and algorithmic composition. Download SC here:

sourceforge.net/projects/supercollider/files/

Numerous helpful links can be found here:
supercollider.sourceforge.net/

You can subscribe to the SC mailing list here, as well as browse the mailing list for questions that may have already been asked and answered:
http://new-supercollider-mailing-lists-forums-use-these.2681727.n2.nabble.com/SuperCollider-Users-New-Use-this-f2676391.html

I also have a series of video tutorials on YouTube:
www.youtube.com/playlist?list=PLPYzvS8A_rTaNDweXe6PX4CXSGq4iEWYC
//////////////////////////////////////////////
//////////////////////////////////////////////
//////////////////////////////////////////////


OBJECT-ORIENTED PROGRAMMING

SC is an object-oriented programming language. Here is an excerpt regarding SC & OOP from a Wesleyan University SC tutorial, which was part of a class I took while I was an SC newbie:

"The SuperCollider programming language belongs to a family of languages that are based on the Object paradigm, a way of conceiving computation, inputs, outputs and data in terms of messages passed between objects.

In this picture, all entities: numbers such as 2, 3, -1, 3.14, individual keyboard entries, known as Characters, collections of such data, such as the Strings "The world is all that is the case", "George Washington", arrays of numbers  [23, 0, -1,22,5], documents, windows, pictures, and sound generators (and so forth), are all Objects.

Each object belongs to a class, whose names are always capitalized.  For example 2, -300 belong to the Integer class, 3.59, 0.01 belong to the Float class, "Hello there" belongs to the String class, [2,3,5], ["Hi", -7, 0.2] belong to the Array class.

Objects send and receive messages. The outcome of such a transaction depends on how the receiving object, called the receiver of a message, interprets the message, or more precisely, how the receiver's class behaves in response to a particular message.

A message has the following syntax in SC:

receiver.message;
"

Don't concern yourself too greatly with the concepts of OOP, but it is useful to have a general understanding of how SC uses the Object paradigm, as well as the receiver.message notation to interpret code and manipulate data.

Please note that "messages" are often referred to as "methods."


//////////////////////////////////////////////
//////////////////////////////////////////////
//////////////////////////////////////////////

INHERITANCE

SC classes are organized in what is usually called the "Class Tree." At the root of the tree is the class of objects called "Object," and all classes are subclasses of Object, downstreat by one or more generations. A unique set of class methods and instance methods are defined for each class, and subclasses inherit all methods defined for its superclass. For example, Array is a subclass of ArrayedCollection, so all methods defined for ArrayedCollection are also defined for Array. Similarly, all methods defined for Object are defined for every class.

//////////////////////////////////////////////
//////////////////////////////////////////////
//////////////////////////////////////////////

EVALUATING CODE

A single line of code is evaluated by placing the cursor anywhere on the line and pressing [enter]. On Mac, this is done by holding [shift] and pressing [return]. Try it on this line:

3.squared;

3 (a member of the Integer class) is the receiver of the method "squared." As expected, the number 9 appears in the post window. If your post window is cluttered, it may be difficult to find the posted result. You can clear the post window by pressing [shift]+[command]+[P].

Multiple lines of code are most easily evaluated when they are enclosed in parentheses, each on its own line, as shown below. To evaluate the following multiple lines with one keystroke, click anywhere within the parentheses an press [command]+[return] (this only works when the code is enclosed in parentheses). Alternatively, you can double click on one of the lines containing a single parenthesis, which will highlight the enclosed code. Then, simply press [shift]+[return] or [command]+[return].

(
3.squared.postln;
3.isPrime.postln;
3+(4*5.0);
)

A default behavior: SC always posts the result of the last line evaluated, so the "postln" method is not needed on the final statement in order to see the result. Also notice that you must end individual statements with a semicolon.

//////////////////////////////////////////////
//////////////////////////////////////////////
//////////////////////////////////////////////

COMMON CLASSES and some of their methods.

You'll undoubetdly use the following classes quite often:

Integer
3.odd; //is this integer odd?
-57.abs; //take the absolute value

Float
//examples
3.14.round; //round to the nearest integer
-90.632.neg; //take the negative of the number (multiply by -1)

String
//strings are enclosed in double quotes.
"Hi, I'm a string.".postln;
"Strings can be reversed".reverse;
"or scrambled".scramble;

Symbol
//it's rare to use methods on Symbols because they're very static. However, methods are defined for them nonetheless. Symbol's immediate superclass is Object.
\mySymbol.postln;
\symb.class; //it's a member of the Symbol class alright.

Array
//An ordered collection of data. SC knows it's an Array because the contents are separated by commas and enclosed in square brackets. Arrays can contain any kinds and any combination of classes/instances. Notice that String and Array share the superclass ArrayedCollection.
[1,2,3,4,5].rotate(1);

x = ["hello", \there, 99];
x.at(1); 	//access the item at index 1 (note that the first item is actually at index 0).
x[1];		//alternative syntax;

x = x.add(555); //append a new item;

x.removeAt(0); 	//remove the item at index 0
//Notice that the "removeAt" method returns the removed item, NOT the remaining array! But sure enough, the 0th item has been removed from our Array 'x':
x;

x.size; //return the size of x

//shortcut syntax for creating large Arrays:
(1..100);

//equivalent to:
Array.series(100,1,1);


Function
//legal SC code, encapsulated in curly braces. Functions can contain multiple lines of code.
f = {2+2};
f.value;

//create arguments and variables for more complex functions:
(
f = {
	arg leg1=3, leg2=4; //arguments with default values
	var sumOfSquares, result; //variables for internal calculation

	//sum the squares and assign to the first variable
	sumOfSquares = (leg1.squared + leg2.squared);

	//take the square root
	result = sumOfSquares.sqrt;

	//Remember that SC always posts the result
	//of the last statement.
};
)

f.value; //evaluate using default values
f.value(5,12); //evaluate using user-supplied values

//of course, we can store the result as a new variable:
g = f.value(7,24);

//and now we've stored the result in the global variable 'g'.

//////////////////////////////////////////////
//////////////////////////////////////////////
//////////////////////////////////////////////

A QUICK WORD ABOUT VARIABLES:

You can use single, lowercase letters as global variables (a,b,c, etc), although by convention, 's' is usually reserved to represent the audio server. You can also precede a variable name with a tilde (~) to make it a global variable. All variables must begin with a lowercase letter (code that begins with an uppercase letter is interpreted as a class). Observe the following examples:


k = 5; //no problem
~myVar = 5; //still no problem
myVar = 5;	//SC throws an error & complains that the variable has not been "declared."

//The following is fine, but myVar is only valid in the context of the 3-statement clump.
(
var myVar;
myVar = 5;
myVar + 4; //return 5+4
)

myVar + 4;	//An error is returned. We are no longer within the initial scope of the declared variable.
