/* SERVER OPTIONS */

s = Server.local.boot;

//ServerOptions is an object that allows the user to retreieve and/or alter (i.e. "get" and "set") information about the audio server, such as the sample block size, the amount of real-time memory that SC is allowed to access, the audio hardware device to be used for input/output, etc. A default instance of ServerOptions is automatically created for any server, and these options can be accessed via the following code:

Server.local.options; //returns the instance of ServerOptions for the local server

s.options; //the same thing

//For example, here we can get the current memSize of the local server
s.options.memSize;

/*
(memSize comes into play when dealing with FFT processing, dynamically allocated delay lines, and other processes in which SC needs to allocate memory on-the-fly. If your processing is too intensive for the current memSize, the audio server will halt signal processing and print "memory allocation failed" errors).
*/

//We can set the memSize like this:
s.options.memSize = 2.pow(20);

//Changes to ServerOptions take effect after the server is rebooted:
s.reboot;

//Now we can see that the memSize attribute has changed:
s.options.memSize;

//You can change the output device using ServerOptions as well. First, let's list all the devices that are available:
ServerOptions.devices;

//Pick the one you want and set it like this:
s.options.outDevice = "Your device's name";

//again, changes take effect upon server reboot:
s.reboot;

//Also useful:

s.options.inDevice; //device used for audio input, which can be different from s.options.outDevice

//if either s.options.outDevice or s.options.inDevice is equal to 'nil', then SC will defer to your operating system for input/output devices (I'm pretty sure about this).

//If your audio interface has, let's say, 30 outputs, you can set the server options to reflect this:
s.options.numOutputBusChannels = 30;

s.reboot;

//display the server level meters, and you can see that this change has indeed taken effect.
s.meter;

//See the ServerOptions help file for more information.