//Trigger Arguments

//When declaring arguments for a SynthDef, or even a simple UGen Function, the user can opt to precede the name of any argument with "t_". Doing so will convert the argument into a control rate trigger. In this case, when set equal to a value of 1, the trigger argument will remain at a value of 1 for exactly one control cycle. On the subsequent control cycle, the trigger argument will return to a value of 0. This is useful, as it allows the user to trigger an envelope multiple times in rapid succession.

ServerTree.removeAll;
s.reboot;

(
SynthDef.new(\trigTest, {
	arg t_gate=0;
	var sig, env;
	env = EnvGen.kr(Env.adsr, t_gate);
	sig = SinOsc.ar * env * 0.5;
	Out.ar(0, sig);
}).add;
)

//Instantiate a Synth. t_gate=0 by default, so initially we hear nothing.
x = Synth(\trigTest);

//Because the envelope's gate control is a trigger argument, we can retrigger the adsr multiple times with this single statement of code. Evaluate this line several times in a row to hear the result.
x.set(\t_gate, 1);

//Free the Synth when you're done.
x.free;


//Compare the previous example to the following example, in which the gate control is a normal argument:
(
SynthDef.new(\trigTest2, {
	arg gate=0;
	var sig, env;
	env = EnvGen.kr(Env.adsr, gate);
	sig = SinOsc.ar * env * 0.5;
	Out.ar(0, sig);
}).add;
)

x = Synth(\trigTest2);

//This is an ordinary argument, so the adsr will be held open until the user sets it back to zero. Evaluating this line more than once in succession will have no effect.
x.set(\gate, 1);

//We can trigger the release portion of the adsr by manually setting the gate argument back to zero:
x.set(\gate, 0);

//And only now can we retrigger the envelope:
x.set(\gate, 1);

x.free;
