//There is a brief demonstration of creating a simple GUI interface at the bottom of this file.

(
ServerTree.removeAll;
~serverSetup = {

SynthDef(\mod, {
	arg freq=8, modType=0, outbus;
	var sig;
	sig = Select.kr(
		which:modType,
		array:[
			LFDNoise0.kr(freq).range(0,1),
			LFDNoise1.kr(freq).range(0,1),
			SinOsc.kr(freq).range(0,1),
			LFSaw.kr(freq).range(0,1),
			LFTri.kr(freq).range(0,1)
		]
	);
	Out.kr(outbus, sig);
}).add;

SynthDef(\trig, {
	arg inbus, freqScale=1, baseFreq=0, trigType=0, len=0.2, outbus;
	var in, sig;
	in = In.kr(inbus) * freqScale + baseFreq;
	sig = Select.kr(
		which:trigType,
		array:[
			Impulse.kr(baseFreq),
			Impulse.kr(in),
			Dust.kr(in)
		]
	);
	sig = Trig.kr(sig, len);
	Out.kr(outbus, sig);
}).add;

SynthDef(\adsr, {
	arg	inbus, t_trig=1, trigType=0, outbus,
		atk=0.01, dec=0.1, slev=0.5, rel=0.6;
	var in, env, trig;
	in = In.kr(inbus);
	trig = Select.kr(
		which:trigType,
		array:[
			t_trig,
			in
		]
	);
	env = EnvGen.kr(Env.adsr(atk,dec,slev,rel),trig);
	Out.kr(outbus, env);
}).add;

SynthDef(\gen, {
	arg	inbusAmp, inbusFreq, inbusWidth, outbus,
		sigType=0, ampType=0, freqType=0, widthType=0,
		amp=1, freqScale=1, baseFreq=440, widthScale=1, baseWidth=0;
	var sig, ampCtrl, freqCtrl, widthCtrl;

	ampCtrl = Select.kr(
		which:ampType,
		array:[
			amp,
			In.kr(inbusAmp)
		]
	);

	freqCtrl = Select.kr(
		which:freqType,
		array:[
			baseFreq,
			In.kr(inbusFreq) * freqScale + baseFreq
		]
	);

	widthCtrl = Select.kr(
		which:widthType,
		array:[
			baseWidth,
			In.kr(inbusWidth) * widthScale + baseWidth
		]
	);
	sig = Select.ar(
		which:sigType,
		array:[
			SinOsc.ar(freqCtrl, 0, ampCtrl),
			VarSaw.ar(freqCtrl, 0, widthCtrl, ampCtrl),
			Pulse.ar(freqCtrl, widthCtrl, ampCtrl),
			WhiteNoise.ar(ampCtrl),
			PinkNoise.ar(ampCtrl),
			BrownNoise.ar(ampCtrl)
		]
	);
	Out.ar(outbus, sig);
}).add;

SynthDef(\filt, {
	arg	inbus1, inbus2, inbus3, inbusCf,
		freqScale=3000, baseFreq=40, filtType=0, outbus;
	var in, cf, sig;
	in = [ In.ar(inbus1), In.ar(inbus2), In.ar(inbus3) ].sum;
	cf = In.kr(inbusCf) * freqScale + baseFreq;
	sig = Select.ar(
		which:filtType,
		array:[
			in,
			LPF.ar(in, cf),
			HPF.ar(in, cf)
		]
	);
	Out.ar(outbus, sig);
}).add;

SynthDef(\out, {
	arg	inbus, inbusPan, outbus=0,
		pan=0, panScale=2, basePan=(-1), panType=0,
		revTime=2.3, mix=0.07, amp=0.75;
	var in, pansig, inPan, sig, verbsig, outsig;
	in = In.ar(inbus);
	inPan = In.kr(inbusPan) * panScale + basePan;
	pansig = Select.kr(
		which:panType,
		array:[
			pan,
			inPan
		]
	);
	sig = Pan2.ar(in, pansig);

	verbsig = sig;
	outsig = 0;
	15.do({
		verbsig = AllpassN.ar(verbsig, 0.06, {Rand(0.001, 0.06)}!2, revTime);
		verbsig = LPF.ar(verbsig, 4500);
		outsig = outsig + verbsig;
	});
	outsig = XFade2.ar(sig, outsig, mix.linlin(0,1,-1,1), amp);
	Out.ar(outbus, outsig);
}).add;

	~modGrp = Group.new;
	~trigGrp = Group.after(~modGrp);
	~envGrp = Group.after(~trigGrp);
	~genGrp = Group.after(~envGrp);
	~filtGrp = Group.after(~genGrp);
	~masterGrp = Group.after(~filtGrp);

	s.newBusAllocators; //free all busses before allocating new ones
	~modOut1 = Bus.control(s);
	~modOut2 = Bus.control(s);
	~modOut3 = Bus.control(s);
	~trigOut = Bus.control(s);
	~envOut = Bus.control(s);
	~genOut1 = Bus.audio(s);
	~genOut2 = Bus.audio(s);
	~genOut3 = Bus.audio(s);
	~filtOut = Bus.audio(s);
};

ServerTree.add(~serverSetup);
s.reboot;
)

//Instantiate Synths
(
~modSynth1 = Synth(
	\mod,
	[
		\outbus, ~modOut1,
		\modType, 2,
		\freq, 0.2
	],
	~modGrp);

~trigSynth = Synth(
	\trig,
	[
		\inbus, ~modOut1,
		\freqScale, 4,
		\baseFreq, 0.1,
		\trigType, 1,
		\outbus, ~trigOut
	],
	~trigGrp);

~envSynth = Synth(
	\adsr,
	[
		\inbus, ~trigOut,
		\trigType, 1,
		\outbus, ~envOut
	],
	~envGrp);

~genSynth = Synth(
	\gen,
	[
		\inbusAmp, ~envOut,
		\inbusFreq, ~modOut1,
		\inbusWidth, ~modOut1,
		\outbus, ~genOut1,
		\sigType, 2,
		\ampType, 1,
		\freqType, 1,
		\widthType, 1,
		\freqScale, 200,
		\baseFreq, 50,
		\widthScale, 0.9,
		\baseWidth, 0.05
	], ~genGrp);

~filtSynth = Synth(
	\filt,
	[
		\inbus1, ~genOut1,
		\inbus2, ~genOut2,
		\inbus3, ~genOut3,
		\inbusCf, ~envOut,
		\filtType, 1,
		\outbus, ~filtOut
	],
	~filtGrp);

~masterSynth = Synth(
	\out,
	[
		\inbus, ~filtOut,
		\inbusPan, ~modOut1,
		\outbus, 0,
		\panType, 1,
		\mix, 0.12,
		\revTime, 3
	],
	~masterGrp);
)


//create a window and make it visible
w = Window.new.front;

//you can close the window like this:
w.close;

//make a new window with a title and specific bounds
w = Window.new("Sloppy GUI", Rect(50,50,700,400)).front;

//place a knob on the window. GUI objects need two arguments: one to provide their parent view (in this case, the Window stored in "w"), and the other to specify their bounds, specified using the Rect class (as was also the case with Windo.new).
~knob = Knob.new(w, Rect.new(40, 40, 100, 100));

//specify an action for the knob, in this case, just posting a random number. Then, click and drag the knob and watch the post window.
(
~knob.action = {
	100.rand.postln;
}
)

//Usually, you'll want to incorporate the value of the knob into the action it performs. This is done by specifying an argument in your GUI action function, which is interpreted as the instance of the GUI object itself. Evaluate this clump, drag the knob around and watch the post window. Most GUI objects range from 0.0 to 1.0 by default.
(
~knob.action = {
	arg thisKnob;
	thisKnob.value.postln;
}
)


//Make sure your Synths have been instantiated (i.e. make sure the sound is still playing). Let's change the ~masterSynth's pan control to read from manually updated user-set values, rather than an upstream signal. You should immediately hear the sound center itself in the stereo field:

~masterSynth.set(\panType, 0);


//Now, redefine the knob's action to set the pan argument of ~masterSynth. We'll have to linearly map the knobs values onto appropriate pan postition values using linlin:
(
~knob.action = {
	arg thisKnob;
	~masterSynth.set(\pan, thisKnob.value.linlin(0,1,-1,1));
}
)

//Now, move the knob around.

//GUI instances have two important methods: value and valueAction. The former will set the value without calling the action function, and the latter will set the value AND call the action function:

//hard left
~knob.valueAction = 0;

//hard right
~knob.valueAction = 1;

//knob is set to 0, but the action function is not called.
~knob.value = 0;

//Note that physically clicking and dragging on a knob (or any other type of GUI) calls valueAction on the GUI object.

//Other cool GUIs (all GUI help files have code examples):
Slider
Button
RangeSlider
PopUpMenu
Slider2D
NumberBox

//text-oriented GUI:
TextView
StaticText
TextField

//enjoy!
s.freeAll;
w.close;