//This code differs only in the inclusion of MIDIdefs at the very bottom (line 1286). If you have your own MIDI controller, you will likely have to modify the expected numbers and values of the MIDIdefs.

s.makeGui;

(
ServerTree.removeAll;

//awaken MIDI
MIDIIn.connectAll;

~serverSetup = {

SynthDef(\mod, {
	arg freq=8, modType=0, outbus;
	var sig;
	sig = Select.kr(
		which:modType,
		array:[
			LFDNoise0.kr(freq).range(0,1),
			LFDNoise1.kr(freq).range(0,1),
			SinOsc.kr(freq).range(0,1),
			LFSaw.kr(freq).range(0,1),
			LFTri.kr(freq).range(0,1)
		]
	);
	Out.kr(outbus, sig);
}).add;

SynthDef(\trig, {
	arg inbus, freqScale=1, baseFreq=0, trigType=0, len=0.2, outbus;
	var in, sig;
	in = In.kr(inbus) * freqScale + baseFreq;
	sig = Select.kr(
		which:trigType,
		array:[
			Impulse.kr(baseFreq),
			Impulse.kr(in),
			Dust.kr(in)
		]
	);
	sig = Trig.kr(sig, len);
	Out.kr(outbus, sig);
}).add;

SynthDef(\adsr, {
	arg	inbus, trig=0, trigType=0, outbus,
		atk=0.01, dec=0.1, slev=0.5, rel=0.2;
	var in, env, trigSig;
	in = In.kr(inbus);
	trigSig = Select.kr(
		which:trigType,
		array:[
			trig,
			in
		]
	);
	env = EnvGen.kr(Env.adsr(atk,dec,slev,rel),trigSig);
	Out.kr(outbus, env);
}).add;

SynthDef(\gen, {
	arg	inbusAmp, inbusFreq, inbusWidth, outbus,
		sigType=0, ampType=0, freqType=0, widthType=0,
		amp=1, freqScale=1, baseFreq=440, widthScale=1, baseWidth=0;
	var sig, ampCtrl, freqCtrl, widthCtrl;

	ampCtrl = Select.kr(
		which:ampType,
		array:[
			amp,
			In.kr(inbusAmp) * amp;
		]
	);

	freqCtrl = Select.kr(
		which:freqType,
		array:[
			baseFreq,
			In.kr(inbusFreq) * freqScale + baseFreq
		]
	);

	widthCtrl = Select.kr(
		which:widthType,
		array:[
			baseWidth,
			In.kr(inbusWidth) * widthScale + baseWidth
		]
	);
	sig = Select.ar(
		which:sigType,
		array:[
			SinOsc.ar(freqCtrl, 0, ampCtrl),
			VarSaw.ar(freqCtrl, 0, widthCtrl, ampCtrl),
			Pulse.ar(freqCtrl, widthCtrl, ampCtrl),
			WhiteNoise.ar(ampCtrl),
			PinkNoise.ar(ampCtrl),
			BrownNoise.ar(ampCtrl)
		]
	);
	Out.ar(outbus, sig);
}).add;

SynthDef(\filt, {
	arg	inbus1, inbus2, inbus3, inbusCf,
		freqScale=3000, baseFreq=40, filtType=0, outbus;
	var in, cf, sig;
	in = [ In.ar(inbus1), In.ar(inbus2), In.ar(inbus3) ].sum;
	cf = In.kr(inbusCf) * freqScale + baseFreq;
	sig = Select.ar(
		which:filtType,
		array:[
			in,
			LPF.ar(in, cf),
			HPF.ar(in, cf)
		]
	);
	Out.ar(outbus, sig);
}).add;

SynthDef(\out, {
	arg	inbus, inbusPan, outbus=0,
		pan=0, panScale=2, basePan=(-1), panType=0,
		revTime=2.3, mix=0.07, amp=0.75;
	var in, pansig, inPan, sig, verbsig, outsig;
	in = In.ar(inbus);
	inPan = In.kr(inbusPan) * panScale + basePan;
	pansig = Select.kr(
		which:panType,
		array:[
			pan,
			inPan
		]
	);
	sig = Pan2.ar(in, pansig);

	verbsig = sig;
	outsig = 0;
	15.do({
		verbsig = AllpassN.ar(verbsig, 0.06, {Rand(0.001, 0.06)}!2, revTime);
		verbsig = LPF.ar(verbsig, 4500);
		outsig = outsig + verbsig;
	});
	outsig = XFade2.ar(sig, outsig, mix.linlin(0,1,-1,1), amp);
	Out.ar(outbus, outsig);
}).add;

	~modGrp = Group.new;
	~trigGrp = Group.after(~modGrp);
	~envGrp = Group.after(~trigGrp);
	~genGrp = Group.after(~envGrp);
	~filtGrp = Group.after(~genGrp);
	~masterGrp = Group.after(~filtGrp);

	s.newBusAllocators; //free all busses before allocating new ones
	~modOut1 = Bus.control(s);
	~modOut2 = Bus.control(s);
	~modOut3 = Bus.control(s);
	~trigOut = Bus.control(s);
	~envOut = Bus.control(s);
	~genOut1 = Bus.audio(s);
	~genOut2 = Bus.audio(s);
	~genOut3 = Bus.audio(s);
	~filtOut = Bus.audio(s);
};

ServerTree.add(~serverSetup);
s.reboot;
)

(
//make Synths
~modSynth1 = Synth(
	\mod,
	[
		\outbus, ~modOut1,
		\modType, 0,
		\freq, 1
	],
	~modGrp);

~modSynth2 = Synth(
	\mod,
	[
		\outbus, ~modOut2,
		\modType, 1,
		\freq, 1
	],
	~modGrp);

~modSynth3 = Synth(
	\mod,
	[
		\outbus, ~modOut3,
		\modType, 2,
		\freq, 1
	],
	~modGrp);

~trigSynth = Synth(
	\trig,
	[
		\inbus, ~modOut1,
		\freqScale, 1,
		\baseFreq, 1,
		\trigType, 1,
		\outbus, ~trigOut
	],
	~trigGrp);

~envSynth = Synth(
	\adsr,
	[
		\inbus, ~trigOut,
		\trigType, 1,
		\outbus, ~envOut
	],
	~envGrp);

~genSynth1 = Synth(
	\gen,
	[
		\inbusAmp, ~envOut,
		\inbusFreq, ~modOut1,
		\inbusWidth, ~modOut1,
		\outbus, ~genOut1,
		\sigType, 0,
		\ampType, 1,
		\freqType, 1,
		\widthType, 1,
		\freqScale, 200,
		\baseFreq, 50,
		\widthScale, 0.9,
		\baseWidth, 0.05
	], ~genGrp);

~genSynth2 = Synth(
	\gen,
	[
		\inbusAmp, ~envOut,
		\inbusFreq, ~modOut1,
		\inbusWidth, ~modOut1,
		\outbus, ~genOut2,
		\sigType, 0,
		\ampType, 0,
		\amp, 0,
		\freqType, 1,
		\widthType, 1,
		\freqScale, 200,
		\baseFreq, 50,
		\widthScale, 0.9,
		\baseWidth, 0.05
	], ~genGrp);

~genSynth3 = Synth(
	\gen,
	[
		\inbusAmp, ~envOut,
		\inbusFreq, ~modOut1,
		\inbusWidth, ~modOut1,
		\outbus, ~genOut3,
		\sigType, 0,
		\ampType, 0,
		\amp, 0,
		\freqType, 1,
		\widthType, 1,
		\freqScale, 200,
		\baseFreq, 50,
		\widthScale, 0.9,
		\baseWidth, 0.05
	], ~genGrp);

~filtSynth = Synth(
	\filt,
	[
		\inbus1, ~genOut1,
		\inbus2, ~genOut2,
		\inbus3, ~genOut3,
		\inbusCf, ~envOut,
		\filtType, 1,
		\outbus, ~filtOut
	],
	~filtGrp);

~masterSynth = Synth(
	\out,
	[
		\inbus, ~filtOut,
		\inbusPan, ~modOut1,
		\outbus, 0,
		\panType, 1,
		\mix, 0.12,
		\revTime, 3,
		\amp, 0
	],
	~masterGrp);




//make GUI
w = Window("modular synth", Rect(100,100,690,630)).front;

w.onClose_({s.freeAll});

~modPanel = CompositeView(w,Rect(0,0,210,280)).background_(Color.rand);
~modPanel.decorator = FlowLayout(~modPanel.bounds, 3@3, 3@9);

StaticText(~modPanel, 200@24)
.string_("Modulators")
.font_(Font("Helvetica", 20))
.align_(\center);

StaticText(~modPanel, 50@24)
.string_("type")
.align_(\center)
.font_(Font("Helvetica", 20));

PopUpMenu(~modPanel, 150@24)
.items_([
	"samp & hold noise",
	"linear noise",
	"LF Sine",
	"LF Saw",
	"LF Tri"
	])
.action_({
	arg thisMenu;
	~modSynth1.set(\modType, thisMenu.value);
});

StaticText(~modPanel, 30@24)
.string_(" hz")
.align_(\center)
.font_(Font("Helvetica", 20));

~mod1freqKnob = Knob(~modPanel, 36@24)
.action_({
	arg thisKnob;
	~modSynth1.set(\freq, thisKnob.value.linexp(0,1,0.05,20));
})
.value_(0.5);

StaticText(~modPanel, 60@24)
.string_("    out")
.font_(Font("Helvetica", 20));


PopUpMenu(~modPanel, 68@24)
.items_([
	"mod1",
	"mod2",
	"mod3"
	])
.action_({
	arg thisMenu;
	case
	{thisMenu.value == 0}
	{~modSynth1.set(\outbus, ~modOut1)}
	{thisMenu.value == 1}
	{~modSynth1.set(\outbus, ~modOut2)}
	{thisMenu.value == 2}
	{~modSynth1.set(\outbus, ~modOut3)}
	{true}{nil};
});

StaticText(~modPanel, 200@12)
.string_("");

StaticText(~modPanel, 50@24)
.string_("type")
.align_(\center)
.font_(Font("Helvetica", 20));

PopUpMenu(~modPanel, 150@24)
.items_([
	"samp & hold noise",
	"linear noise",
	"LF Sine",
	"LF Saw",
	"LF Tri"
	])
.value_(1)
.action_({
	arg thisMenu;
	~modSynth2.set(\modType, thisMenu.value);
});

StaticText(~modPanel, 30@24)
.string_(" hz")
.align_(\center)
.font_(Font("Helvetica", 20));

Knob(~modPanel, 36@24)
.action_({
	arg thisKnob;
	~modSynth2.set(\freq, thisKnob.value.linexp(0,1,0.05,20));
})
.value_(0.5);

StaticText(~modPanel, 60@24)
.string_("    out")
.font_(Font("Helvetica", 20));


PopUpMenu(~modPanel, 68@24)
.items_([
	"mod1",
	"mod2",
	"mod3"
	])
.action_({
	arg thisMenu;
	case
	{thisMenu.value == 0}
	{~modSynth2.set(\outbus, ~modOut1)}
	{thisMenu.value == 1}
	{~modSynth2.set(\outbus, ~modOut2)}
	{thisMenu.value == 2}
	{~modSynth2.set(\outbus, ~modOut3)}
	{true}{nil};
})
.value_(1);

StaticText(~modPanel, 200@12)
.string_("");

StaticText(~modPanel, 50@24)
.string_("type")
.align_(\center)
.font_(Font("Helvetica", 20));

PopUpMenu(~modPanel, 150@24)
.items_([
	"samp & hold noise",
	"linear noise",
	"LF Sine",
	"LF Saw",
	"LF Tri"
	])
.value_(2)
.action_({
	arg thisMenu;
	~modSynth3.set(\modType, thisMenu.value);
});

StaticText(~modPanel, 30@24)
.string_(" hz")
.align_(\center)
.font_(Font("Helvetica", 20));

Knob(~modPanel, 36@24)
.action_({
	arg thisKnob;
	~modSynth3.set(\freq, thisKnob.value.linexp(0,1,0.05,20));
})
.value_(0.5);

StaticText(~modPanel, 60@24)
.string_("    out")
.font_(Font("Helvetica", 20));


PopUpMenu(~modPanel, 68@24)
.items_([
	"mod1",
	"mod2",
	"mod3"
	])
.action_({
	arg thisMenu;
	case
	{thisMenu.value == 0}
	{~modSynth3.set(\outbus, ~modOut1)}
	{thisMenu.value == 1}
	{~modSynth3.set(\outbus, ~modOut2)}
	{thisMenu.value == 2}
	{~modSynth3.set(\outbus, ~modOut3)}
	{true}{nil};
})
.value_(2);

~trigPanel = CompositeView(w,Rect(0,280,210,190)).background_(Color.rand);
~trigPanel.decorator = FlowLayout(~trigPanel.bounds, 3@3, 3@12);

StaticText(~trigPanel, 200@24)
.string_("Triggers")
.font_(Font("Helvetica", 20))
.align_(\center);

StaticText(~trigPanel, 50@24)
.string_("type")
.align_(\center)
.font_(Font("Helvetica", 20));

PopUpMenu(~trigPanel, 150@24)
.items_([
	"basic impulse",
	"signal impulse",
	"signal dust"
	])
.action_({
	arg thisMenu;
	~trigSynth.set(\trigType, thisMenu.value);
})
.value_(1);

StaticText(~trigPanel, 60@28)
.string_("base Hz")
.align_(\center)
.font_(Font("Helvetica", 14));

Knob(~trigPanel, 36@24)
.action_({
	arg thisKnob;
	~trigSynth.set(\baseFreq, thisKnob.value.linexp(0,1,0.05,20));
})
.value_(0.5);

StaticText(~trigPanel, 60@20)
.string_("Hz range")
.align_(\center)
.font_(Font("Helvetica", 14));

Knob(~trigPanel, 36@24)
.action_({
	arg thisKnob;
	~trigSynth.set(\freqScale, thisKnob.value.linlin(0,1,0,20));
})
.value_(0.05);

StaticText(~trigPanel, 50@28)
.string_("trig dur")
.align_(\center)
.font_(Font("Helvetica", 14));

Slider(~trigPanel, 150@20)
.action_({
	arg thisSlider;
	~trigSynth.set(\len, thisSlider.value.linexp(0,1,0.005,10));
})
.value_(0.2.explin(0.005,10,0,1));


StaticText(~trigPanel, 30@28)
.string_("in")
.align_(\center)
.font_(Font("Helvetica", 16));

PopUpMenu(~trigPanel, 68@24)
.items_([
	"mod1",
	"mod2",
	"mod3"
	])
.action_({
	arg thisMenu;
	case
	{thisMenu.value == 0}
	{~trigSynth.set(\inbus, ~modOut1)}
	{thisMenu.value == 1}
	{~trigSynth.set(\inbus, ~modOut2)}
	{thisMenu.value == 2}
	{~trigSynth.set(\inbus, ~modOut3)}
	{true}{nil};
});

StaticText(~trigPanel, 30@28)
.string_("out")
.align_(\center)
.font_(Font("Helvetica", 16));

PopUpMenu(~trigPanel, 50@24)
.items_([
	"trig"
	])
.action_({
	arg thisMenu;
	~trigSynth.set(\outbus, ~trigOut)
});

~adsrPanel = CompositeView(w,Rect(0,470,210,160)).background_(Color.rand);
~adsrPanel.decorator = FlowLayout(~adsrPanel.bounds, 3@3, 3@12);

StaticText(~adsrPanel, 200@24)
.string_("ADSR Envelope")
.font_(Font("Helvetica", 20))
.align_(\center);

StaticText(~adsrPanel, 46@24)
.string_("type")
.align_(\center)
.font_(Font("Helvetica", 20));

PopUpMenu(~adsrPanel, 80@24)
.items_([
	"manual",
	"signal"
	])
.action_({
	arg thisMenu;
	~envSynth.set(\trigType, thisMenu.value);
})
.value_(1);

StaticText(~adsrPanel, 24@24).string_("");

~manualTrigButton = Button(~adsrPanel, 24@24)
.states_([["T"]])
.action_({nil});

StaticText(~adsrPanel, 20@28)
.string_("in")
.align_(\center)
.font_(Font("Helvetica", 16));

PopUpMenu(~adsrPanel, 68@24)
.items_([
	"mod1",
	"mod2",
	"mod3",
	"trig"
	])
.action_({
	arg thisMenu;
	case
	{thisMenu.value == 0}
	{~envSynth.set(\inbus, ~modOut1)}
	{thisMenu.value == 1}
	{~envSynth.set(\inbus, ~modOut2)}
	{thisMenu.value == 2}
	{~envSynth.set(\inbus, ~modOut3)}
	{thisMenu.value == 3}
	{~envSynth.set(\inbus, ~trigOut)}
	{true}{nil};
})
.value_(3);

StaticText(~adsrPanel, 40@28)
.string_("   out")
.align_(\center)
.font_(Font("Helvetica", 16));

PopUpMenu(~adsrPanel, 63@24)
.items_([
	"adsr"
	])
.action_({
	arg thisMenu;
	~envSynth.set(\outbus, ~envOut)
});

StaticText(~adsrPanel, 38@30).string_("asdr").align_(\center);

NumberBox(~adsrPanel, 38@24)
.action_({
	arg thisBox;
	~envSynth.set(\atk, thisBox.value);
})
.value_(0.01)
.clipLo_(0.001)
.clipHi_(20);

NumberBox(~adsrPanel, 38@24)
.action_({
	arg thisBox;
	~envSynth.set(\dec, thisBox.value);
})
.value_(0.1)
.clipLo_(0.001)
.clipHi_(20);

NumberBox(~adsrPanel, 38@24)
.action_({
	arg thisBox;
	~envSynth.set(\slev, thisBox.value);
})
.value_(0.5)
.clipLo_(0.0)
.clipHi_(1.0);

NumberBox(~adsrPanel, 38@24)
.action_({
	arg thisBox;
	~envSynth.set(\rel, thisBox.value);
})
.value_(0.2)
.clipLo_(0.001)
.clipHi_(20);

~genPanel = CompositeView(w,Rect(210,0,240,500)).background_(Color.rand);
~genPanel.decorator = FlowLayout(~genPanel.bounds, 3@3, 3@3);

StaticText(~genPanel, 240@24)
.string_("Signal Generators")
.font_(Font("Helvetica", 20))
.align_(\center);

StaticText(~genPanel, 30@24)
.string_("type");

PopUpMenu(~genPanel, 200@24)
.items_([\sine, \varsaw, \pulse, \white, \pink, \brown])
.action_({
	arg thisMenu;
	~genSynth1.set(\sigType, thisMenu.value);
});

StaticText(~genPanel, 30@24)
.string_("amp");

PopUpMenu(~genPanel, 80@24)
.items_([\manual, \signal])
.action_({
	arg thisMenu;
	~genSynth1.set(\ampType, thisMenu.value);
})
.value_(1);

PopUpMenu(~genPanel, 90@24)
.items_([\mod1, \mod2, \mod3, \trig, \adsr])
.action_({
	arg thisMenu;
	case
	{thisMenu.value == 0}
	{~genSynth1.set(\inbusAmp, ~modOut1)}
	{thisMenu.value == 1}
	{~genSynth1.set(\inbusAmp, ~modOut2)}
	{thisMenu.value == 2}
	{~genSynth1.set(\inbusAmp, ~modOut3)}
	{thisMenu.value == 3}
	{~genSynth1.set(\inbusAmp, ~trigOut)}
	{thisMenu.value == 4}
	{~genSynth1.set(\inbusAmp, ~envOut)}
})
.value_(4);

Knob(~genPanel, 24@24)
.action_({
	arg thisKnob;
	~genSynth1.set(\amp, thisKnob.value);
})
.value_(1);

StaticText(~genPanel, 30@24)
.string_("freq");

PopUpMenu(~genPanel, 80@24)
.items_([\manual, \signal])
.action_({
	arg thisMenu;
	~genSynth1.set(\freqType, thisMenu.value);
})
.value_(1);

PopUpMenu(~genPanel, 62@24)
.items_([\mod1, \mod2, \mod3, \trig, \adsr])
.action_({
	arg thisMenu;
	thisMenu.postln;
	case
	{thisMenu.value == 0}
	{~genSynth1.set(\inbusFreq, ~modOut1)}
	{thisMenu.value == 1}
	{~genSynth1.set(\inbusFreq, ~modOut2)}
	{thisMenu.value == 2}
	{~genSynth1.set(\inbusFreq, ~modOut3)}
	{thisMenu.value == 3}
	{~genSynth1.set(\inbusFreq, ~trigOut)}
	{thisMenu.value == 4}
	{~genSynth1.set(\inbusFreq, ~envOut)}
})
.value_(0);

~gen1baseFreqKnob = Knob(~genPanel, 24@24)
.action_({
	arg thisKnob;
	~genSynth1.set(\baseFreq, thisKnob.value.linexp(0,1,20,16000));
})
.value_(50.explin(20,16000,0,1));

~gen1freqScaleKnob = Knob(~genPanel, 24@24)
.action_({
	arg thisKnob;
	~genSynth1.set(\freqScale, thisKnob.value.linlin(0,1,0,5000));
})
.value_(200.linlin(0,5000,0,1));

StaticText(~genPanel, 30@24)
.string_("wdth");

PopUpMenu(~genPanel, 80@24)
.items_([\manual, \signal])
.action_({
	arg thisMenu;
	~genSynth1.set(\widthType, thisMenu.value);
})
.value_(1);

PopUpMenu(~genPanel, 62@24)
.items_([\mod1, \mod2, \mod3, \trig, \adsr])
.action_({
	arg thisMenu;
	case
	{thisMenu.value == 0}
	{~genSynth1.set(\inbusWidth, ~modOut1)}
	{thisMenu.value == 1}
	{~genSynth1.set(\inbusWidth, ~modOut2)}
	{thisMenu.value == 2}
	{~genSynth1.set(\inbusWidth, ~modOut3)}
	{thisMenu.value == 3}
	{~genSynth1.set(\inbusWidth, ~trigOut)}
	{thisMenu.value == 4}
	{~genSynth1.set(\inbusWidth, ~envOut)}
})
.value_(0);

Knob(~genPanel, 24@24)
.action_({
	arg thisKnob;
	~genSynth1.set(\baseWidth, thisKnob.value.clip(0.001,0.999));
})
.value_(0.05);

Knob(~genPanel, 24@24)
.action_({
	arg thisKnob;
	~genSynth1.set(\widthScale, thisKnob.value)
})
.value(0.9);

StaticText(~genPanel, 30@24)
.string_("out");

PopUpMenu(~genPanel, 62@24)
.items_([\gen1, \gen2, \gen3, \filt])
.action_({
	arg thisMenu;
	case
	{thisMenu.value == 0}
	{~genSynth1.set(\outbus, ~genOut1)}
	{thisMenu.value == 1}
	{~genSynth1.set(\outbus, ~genOut2)}
	{thisMenu.value == 2}
	{~genSynth1.set(\outbus, ~genOut3)}
	{thisMenu.value == 3}
	{~genSynth1.set(\outbus, ~filtOut)};
});

StaticText(~genPanel, 210@24)
.string_("");

StaticText(~genPanel, 30@24)
.string_("type");

PopUpMenu(~genPanel, 200@24)
.items_([\sine, \varsaw, \pulse, \white, \pink, \brown])
.action_({
	arg thisMenu;
	~genSynth2.set(\sigType, thisMenu.value);
});

StaticText(~genPanel, 30@24)
.string_("amp");

PopUpMenu(~genPanel, 80@24)
.items_([\manual, \signal])
.action_({
	arg thisMenu;
	~genSynth2.set(\ampType, thisMenu.value);
});

PopUpMenu(~genPanel, 90@24)
.items_([\mod1, \mod2, \mod3, \trig, \adsr])
.action_({
	arg thisMenu;
	case
	{thisMenu.value == 0}
	{~genSynth2.set(\inbusAmp, ~modOut1)}
	{thisMenu.value == 1}
	{~genSynth2.set(\inbusAmp, ~modOut2)}
	{thisMenu.value == 2}
	{~genSynth2.set(\inbusAmp, ~modOut3)}
	{thisMenu.value == 3}
	{~genSynth2.set(\inbusAmp, ~trigOut)}
	{thisMenu.value == 4}
	{~genSynth2.set(\inbusAmp, ~envOut)}
});

Knob(~genPanel, 24@24)
.action_({
	arg thisKnob;
	~genSynth2.set(\amp, thisKnob.value);
});

StaticText(~genPanel, 30@24)
.string_("freq");

PopUpMenu(~genPanel, 80@24)
.items_([\manual, \signal])
.action_({
	arg thisMenu;
	~genSynth2.set(\freqType, thisMenu.value);
});

PopUpMenu(~genPanel, 62@24)
.items_([\mod1, \mod2, \mod3, \trig, \adsr])
.action_({
	arg thisMenu;
	case
	{thisMenu.value == 0}
	{~genSynth2.set(\inbusFreq, ~modOut1)}
	{thisMenu.value == 1}
	{~genSynth2.set(\inbusFreq, ~modOut2)}
	{thisMenu.value == 2}
	{~genSynth2.set(\inbusFreq, ~modOut3)}
	{thisMenu.value == 3}
	{~genSynth2.set(\inbusFreq, ~trigOut)}
	{thisMenu.value == 4}
	{~genSynth2.set(\inbusFreq, ~envOut)}
});

Knob(~genPanel, 24@24)
.action_({
	arg thisKnob;
	~genSynth2.set(\baseFreq, thisKnob.value.linexp(0,1,20,16000));
})
.value_(440.explin(20,16000,0,1));

Knob(~genPanel, 24@24)
.action_({
	arg thisKnob;
	~genSynth2.set(\freqScale, thisKnob.value.linlin(0,1,0,5000));
})
.value_(0);

StaticText(~genPanel, 30@24)
.string_("wdth");

PopUpMenu(~genPanel, 80@24)
.items_([\manual, \signal])
.action_({
	arg thisMenu;
	~genSynth2.set(\widthType, thisMenu.value);
});

PopUpMenu(~genPanel, 62@24)
.items_([\mod1, \mod2, \mod3, \trig, \adsr])
.action_({
	arg thisMenu;
	case
	{thisMenu.value == 0}
	{~genSynth2.set(\inbusWidth, ~modOut1)}
	{thisMenu.value == 1}
	{~genSynth2.set(\inbusWidth, ~modOut2)}
	{thisMenu.value == 2}
	{~genSynth2.set(\inbusWidth, ~modOut3)}
	{thisMenu.value == 3}
	{~genSynth2.set(\inbusWidth, ~trigOut)}
	{thisMenu.value == 4}
	{~genSynth2.set(\inbusWidth, ~envOut)}
});

Knob(~genPanel, 24@24)
.action_({
	arg thisKnob;
	~genSynth2.set(\baseWidth, thisKnob.value.clip(0.001,0.999));
})
.value_(0.05);

Knob(~genPanel, 24@24)
.action_({
	arg thisKnob;
	~genSynth2.set(\widthScale, thisKnob.value)
})
.value(0.9);

StaticText(~genPanel, 30@24)
.string_("out");

PopUpMenu(~genPanel, 62@24)
.items_([\gen1, \gen2, \gen3, \filt])
.action_({
	arg thisMenu;
	case
	{thisMenu.value == 0}
	{~genSynth2.set(\outbus, ~genOut1)}
	{thisMenu.value == 1}
	{~genSynth2.set(\outbus, ~genOut2)}
	{thisMenu.value == 2}
	{~genSynth2.set(\outbus, ~genOut3)}
	{thisMenu.value == 3}
	{~genSynth1.set(\outbus, ~filtOut)};
})
.value_(1);

StaticText(~genPanel, 210@24)
.string_("");

StaticText(~genPanel, 30@24)
.string_("type");

PopUpMenu(~genPanel, 200@24)
.items_([\sine, \varsaw, \pulse, \white, \pink, \brown])
.action_({
	arg thisMenu;
	~genSynth3.set(\sigType, thisMenu.value);
});

StaticText(~genPanel, 30@24)
.string_("amp");

PopUpMenu(~genPanel, 80@24)
.items_([\manual, \signal])
.action_({
	arg thisMenu;
	~genSynth3.set(\ampType, thisMenu.value);
});

PopUpMenu(~genPanel, 90@24)
.items_([\mod1, \mod2, \mod3, \trig, \adsr])
.action_({
	arg thisMenu;
	case
	{thisMenu.value == 0}
	{~genSynth3.set(\inbusAmp, ~modOut1)}
	{thisMenu.value == 1}
	{~genSynth3.set(\inbusAmp, ~modOut2)}
	{thisMenu.value == 2}
	{~genSynth3.set(\inbusAmp, ~modOut3)}
	{thisMenu.value == 3}
	{~genSynth3.set(\inbusAmp, ~trigOut)}
	{thisMenu.value == 4}
	{~genSynth3.set(\inbusAmp, ~envOut)}
});

Knob(~genPanel, 24@24)
.action_({
	arg thisKnob;
	~genSynth3.set(\amp, thisKnob.value);
});

StaticText(~genPanel, 30@24)
.string_("freq");

PopUpMenu(~genPanel, 80@24)
.items_([\manual, \signal])
.action_({
	arg thisMenu;
	~genSynth3.set(\freqType, thisMenu.value);
});

PopUpMenu(~genPanel, 62@24)
.items_([\mod1, \mod2, \mod3, \trig, \adsr])
.action_({
	arg thisMenu;
	case
	{thisMenu.value == 0}
	{~genSynth3.set(\inbusFreq, ~modOut1)}
	{thisMenu.value == 1}
	{~genSynth3.set(\inbusFreq, ~modOut2)}
	{thisMenu.value == 2}
	{~genSynth3.set(\inbusFreq, ~modOut3)}
	{thisMenu.value == 3}
	{~genSynth3.set(\inbusFreq, ~trigOut)}
	{thisMenu.value == 4}
	{~genSynth3.set(\inbusFreq, ~envOut)}
});

Knob(~genPanel, 24@24)
.action_({
	arg thisKnob;
	~genSynth3.set(\baseFreq, thisKnob.value.linexp(0,1,20,16000));
})
.value_(440.explin(20,16000,0,1));

Knob(~genPanel, 24@24)
.action_({
	arg thisKnob;
	~genSynth3.set(\freqScale, thisKnob.value.linlin(0,1,0,5000));
})
.value_(0);

StaticText(~genPanel, 30@24)
.string_("wdth");

PopUpMenu(~genPanel, 80@24)
.items_([\manual, \signal])
.action_({
	arg thisMenu;
	~genSynth3.set(\widthType, thisMenu.value);
});

PopUpMenu(~genPanel, 62@24)
.items_([\mod1, \mod2, \mod3, \trig, \adsr])
.action_({
	arg thisMenu;
	case
	{thisMenu.value == 0}
	{~genSynth3.set(\inbusWidth, ~modOut1)}
	{thisMenu.value == 1}
	{~genSynth3.set(\inbusWidth, ~modOut2)}
	{thisMenu.value == 2}
	{~genSynth3.set(\inbusWidth, ~modOut3)}
	{thisMenu.value == 3}
	{~genSynth3.set(\inbusWidth, ~trigOut)}
	{thisMenu.value == 4}
	{~genSynth3.set(\inbusWidth, ~envOut)}
});

Knob(~genPanel, 24@24)
.action_({
	arg thisKnob;
	~genSynth3.set(\baseWidth, thisKnob.value.clip(0.001,0.999));
})
.value_(0.05);

Knob(~genPanel, 24@24)
.action_({
	arg thisKnob;
	~genSynth3.set(\widthScale, thisKnob.value)
})
.value(0.9);

StaticText(~genPanel, 30@24)
.string_("out");

PopUpMenu(~genPanel, 62@24)
.items_([\gen1, \gen2, \gen3, \filt])
.action_({
	arg thisMenu;
	case
	{thisMenu.value == 0}
	{~genSynth3.set(\outbus, ~genOut1)}
	{thisMenu.value == 1}
	{~genSynth3.set(\outbus, ~genOut2)}
	{thisMenu.value == 2}
	{~genSynth3.set(\outbus, ~genOut3)}
	{thisMenu.value == 3}
	{~genSynth1.set(\outbus, ~filtOut)};
});

StaticText(~genPanel, 200@24)
.string_("");

~filtPanel = CompositeView(w,Rect(210,500,240,130)).background_(Color.rand);
~filtPanel.decorator = FlowLayout(~filtPanel.bounds, 3@3, 3@9);

StaticText(~filtPanel, 240@24)
.string_("Filter")
.font_(Font("Helvetica", 20))
.align_(\center);

StaticText(~filtPanel, 30@24)
.string_("type");

PopUpMenu(~filtPanel, 200@24)
.items_([\none, \LPF, \HPF])
.action_({
	arg thisMenu;
	~filtSynth.set(\filtType, thisMenu.value);
})
.value_(1);

StaticText(~filtPanel, 30@24)
.string_("cf");

PopUpMenu(~filtPanel, 130@24)
.items_([\mod1, \mod2, \mod3, \trig, \adsr])
.action_({
	arg thisMenu;
	case
	{thisMenu.value == 0}
	{~filtSynth.set(\inbusCf, ~modOut1)}
	{thisMenu.value == 1}
	{~filtSynth.set(\inbusCf, ~modOut2)}
	{thisMenu.value == 2}
	{~filtSynth.set(\inbusCf, ~modOut3)}
	{thisMenu.value == 3}
	{~filtSynth.set(\inbusCf, ~trigOut)}
	{thisMenu.value == 4}
	{~filtSynth.set(\inbusCf, ~envOut)}
})
.value_(4);

Knob(~filtPanel, 24@24)
.action_({
	arg thisKnob;
	~filtSynth.set(\baseFreq, thisKnob.value.linexp(0,1,20,15000));
})
.value_(40.explin(20,15000,0,1));

Knob(~filtPanel, 24@24)
.action_({
	arg thisKnob;
	~filtSynth.set(\freqScale, thisKnob.value.linlin(0,1,0,5000));
})
.value_(0.6);

~outPanel = CompositeView(w,Rect(450,0,240,630)).background_(Color.rand);
~outPanel.decorator = FlowLayout(~outPanel.bounds, 3@3, 3@3);

StaticText(~outPanel, 240@28)
.string_("Main Out")
.font_(Font("Helvetica", 20))
.align_(\center);

StaticText(~outPanel, 35@28)
.string_("amp")
.align_(\center)
.font_(Font("Helvetica", 14));

~outAmpSlider = Slider(~outPanel, 190@20)
.action_({
	arg thisSlider;
	~masterSynth.set(\amp, thisSlider.value);
})
.value_(0);

StaticText(~outPanel, 50@28)
.string_("rev mix")
.align_(\center)
.font_(Font("Helvetica", 14));

~outRevMixSlider = Slider(~outPanel, 175@20)
.action_({
	arg thisSlider;
	~masterSynth.set(\mix, thisSlider.value);
})
.value_(0.12);

StaticText(~outPanel, 50@28)
.string_("rev time")
.align_(\center)
.font_(Font("Helvetica", 14));

Slider(~outPanel, 175@20)
.action_({
	arg thisSlider;
	~masterSynth.set(\revTime, thisSlider.value.linexp(0,1,0.6,5));
})
.value_(3.explin(0.6,5,0,1));

StaticText(~outPanel, 30@28)
.string_("pan")
.align_(\center)
.font_(Font("Helvetica", 14));

PopUpMenu(~outPanel, 100@24)
.items_([\manual, \signal])
.action_({
	arg thisMenu;
	~masterSynth.set(\panType, thisMenu.value);
})
.value_(1);

PopUpMenu(~outPanel, 70@24)
.items_([\mod1, \mod2, \mod3, \trig, \adsr])
.action_({
	arg thisMenu;
	case
	{thisMenu.value == 0}
	{~masterSynth.set(\inbusPan, ~modOut1)}
	{thisMenu.value == 1}
	{~masterSynth.set(\inbusPan, ~modOut2)}
	{thisMenu.value == 2}
	{~masterSynth.set(\inbusPan, ~modOut3)}
	{thisMenu.value == 3}
	{~masterSynth.set(\inbusPan, ~trigOut)}
	{thisMenu.value == 4}
	{~masterSynth.set(\inbusPan, ~envOut)}
})
.value_(0);

RangeSlider(~outPanel, 234@30)
.action_({
	arg thisRanger;
	~masterSynth.set(\pan, (thisRanger.lo + (thisRanger.range/2)).linlin(0,1,-1,1));
	~masterSynth.set(\basePan, thisRanger.lo.linlin(0,1,-1,1));
	~masterSynth.set(\panScale, thisRanger.range);
});




//make MIDIdefs
MIDIdef.cc(\updateGUI, {
	arg val, num, chan, src;
	case
	{num == 18}
	{{~outRevMixSlider.valueAction_(val.linlin(0,127,0,1))}.defer}
	{num == 19}
	{{~outAmpSlider.valueAction_(val.linlin(0,127,0,1))}.defer}
	{num == 27}
	{{~mod1freqKnob.valueAction_(val.linlin(0,127,0,1))}.defer}
	{num == 28}
	{{~gen1baseFreqKnob.valueAction_(val.linlin(0,127,0,1))}.defer}
	{num == 29}
	{{~gen1freqScaleKnob.valueAction_(val.linlin(0,127,0,1))}.defer}
	{true}{nil};
});

MIDIdef.noteOn(\manualNoteTrig, {
	arg val, num, chan, src;
	case
	{chan == 1}
	{~envSynth.set(\trig, 1)}
	{true}{nil};
});

MIDIdef.noteOff(\manualNoteTrigOff, {
	arg val, num, chan, src;
	case
	{chan == 1}
	{~envSynth.set(\trig, 0)}
	{true}{nil};
});
)