s.boot;

(
ServerTree.removeAll;

~makeNodes = {
	z = Synth(\reverb, [\in, ~reverbBus]);
	y = Synth(\tone, [\out, ~reverbBus]);
};

ServerTree.add(~makeNodes);

~reverbBus = Bus.audio(s, 2);

SynthDef(\reverb, {
	arg in, out=0, revTime=1.8, mix=0.06, amp=0.8;
	var sig, dry, wet, outsig;
	sig = In.ar(in, 2);
	dry = sig;
	wet = 0;
	12.do{
		sig = AllpassN.ar(sig, 0.06, {Rand(0.001, 0.06)}!2, revTime);
		sig = LPF.ar(sig, 4500);
		wet = wet + sig;
	};
	outsig = XFade2.ar(dry, wet, mix.linlin(0,1,-1,1), amp);
	Out.ar(out, outsig);
}).add;

SynthDef(\tone, {
	arg out, freq=440, amp=0.85, t_trig=0, atk=0.01, dec=0.02, rel=0.1, slev=0.3, pan=0;
	var env, sig;
	env = EnvGen.kr(Env.new([0,1,slev,0],[atk,dec,rel],[1,-1,-1]), t_trig);
	sig = SinOsc.ar(freq);
	sig = sig * env;
	sig = Pan2.ar(sig, pan, amp);
	Out.ar(out, sig);
}).add;

s.reboot;
)



//Tasks: algorithmic language-side control of Nodes on the Server

//a sequential Task

(
t = Task.new({
	y.set(\t_trig, 1, \freq, 440, \pan, 0);
	wait(1);
	y.set(\t_trig, 1, \freq, 880, \pan, 0);
	wait(0.5);
	y.set(\t_trig, 1, \freq, 220, \pan, -1);
	wait(0.25);
	y.set(\t_trig, 1, \freq, 330, \pan, 1);
}).start;
)

t.start;
t.stop;
t.resume;



//an infinite loop within a Task
(
t = Task.new({
	{
		y.set(\t_trig, 1, \freq, 440, \pan, 0);
		wait(1);
	}.loop;
}).start;
)

t.stop;



//a finite loop within a Task

(
t = Task.new({
	5.do{
		y.set(\t_trig, 1, \freq, 440, \pan, 0);
		wait(1);
	};
}).start;
)



//random values

(
t = Task.new({
	{
		y.set(\t_trig, 1, \freq, exprand(100,1000).round(100), \pan, rrand(-1.0,1.0));
		wait(exprand(0.1,1.25));
	}.loop;
}).start;
)

t.stop;



//selecting randomly from a collection of values
(
t = Task.new({
	{
		y.set(
			\t_trig, 1,
			\freq, [200,400,500,600].choose,
			\pan, [-1,0,1].choose
		);
		wait([0.125,0.25,0.5].choose);
	}.loop;
}).start;
)

t.stop;



//the same, with weighted choice
(
t = Task.new({
	{
		y.set(
			\t_trig, 1,
			\freq, [200,400,500,600,700].choose,
			\pan, [-1,0,1].wchoose([0.45,0.1,0.45]),
			\amp, exprand(0.02,0.8),
			\rel, exprand(0.01,0.4)
		);
		wait([0.125,0.25,0.5].wchoose([0.8,0.15,0.05]));
	}.loop;
}).start;
)

t.stop;



//selecting randomly from a range of values
(
t = Task.new({
	{
		y.set(
			\amp, exprand(0.05,0.85),
			\freq, rrand(38,85).midicps, //chromatic
			\pan, rrand(-0.8,0.8),
			\t_trig, 1
		);
		exprand(0.08,1).wait;
	}.loop;
}).start;
)

t.stop;



//even more parameters
(
t = Task.new({
	{
		y.set(
			\amp, 		exprand(0.05,0.85),
			\freq,		rrand(38.0,85.0).midicps, //microtonal
			\pan,		rrand(-0.8,0.8),
			\atk,		exprand(0.01,0.2),
			\dec,		exprand(0.01,0.2),
			\rel,		exprand(0.01,0.5),
			\slev,		exprand(0.01,0.5),
			\t_trig,	1
		);
		exprand(0.1,1.5).wait;
	}.loop;
}).start;
)

t.stop;

s.freeAll;

//Patterns
//Pseq - sequential value output
(
y.set(\atk, 0.01, \dec, 0.05, \rel, 0.1, \amp, 0.6); //reset parameters
~freq = Pseq([ 100, 200, 300, 400, 500, 600, 700, 800, 900, 1000 ], inf).asStream;
t = Task.new({
	{
		y.set(\t_trig, 1, \freq, ~freq.next, \pan, rrand(-0.6,0.6));
		wait(0.17);
	}.loop;
}).start;
)

t.stop;



//Prand - randomly choose from a collection of values
(
~waitTimes = Prand([
	Pseq([0.125],4),
	Pseq([0.25],2),
	Pseq([0.5],1)
], inf).asStream;

~freq = Pseq([ 100, 200, 300, 400, 500, 600, 700, 800, 900, 1000 ], inf).asStream;

t = Task.new({
	{
		y.set(\t_trig, 1, \freq, ~freq.next, \pan, rrand(-1.0,1.0));
		wait(~waitTimes.next);
	}.loop;
}).start;
)

t.stop;



//Pxrand - randomly choose from a collection of values, but never choose the same value twice in a row
(
~waitTimes = Pxrand([
	Pseq([0.125],4),
	Pseq([0.25],2),
	Pseq([0.5],1)
], inf).asStream;

~freq = Pseq([ 100, 200, 300, 400, 500, 600, 700, 800, 900, 1000 ], inf).asStream;

t = Task.new({
	{
		y.set(\t_trig, 1, \freq, ~freq.next, \pan, rrand(-1.0,1.0));
		wait(~waitTimes.next);
	}.loop;
}).start;
)

t.stop;



//Pwrand - weighted random choices
(
~waitTimes = Pxrand([
	Pseq([0.125],4),
	Pseq([0.25],2),
	Pseq([0.5],1)
], inf).asStream;

//favor higher frequencies
~freq = Pwrand([ 100, 200, 300, 400, 500 ], [1,5,10,20,30].normalizeSum, inf).asStream;

t = Task.new({
	{
		y.set(\t_trig, 1, \freq, ~freq.next, \pan, rrand(-1.0,1.0));
		wait(~waitTimes.next);
	}.loop;
}).start;
)

t.stop;



//Pshuf - cycle through the values in a collection in a fixed, randomly chosen order
(
~freq = Pshuf([ 100, 200, 300, 400, 500, 600, 700, 800, 900, 1000 ], inf).asStream;

t = Task.new({
	{
		y.set(\t_trig, 1, \freq, ~freq.next.postln, \pan, rrand(-1.0,1.0));
		wait(0.125);
	}.loop;
}).start;
)

t.stop;



//Pwhite - random value between a min and max
(
~waitTimes = Pxrand([
	Pseq([0.125],4),
	Pseq([0.25],2),
	Pseq([0.5],1)
], inf).asStream;

~freq = Pwhite(40,80,inf).asStream; //only integers, since min and max are both integers

t = Task.new({
	{
		y.set(\t_trig, 1, \freq, ~freq.next.postln.midicps, \pan, rrand(-1.0,1.0));
		wait(~waitTimes.next);
	}.loop;
}).start;
)

t.stop;


//See the help files on the following additional Patterns if you're curious:

//Pbrown

//Pstutter

//Pgeom

//Pseries

//Plprand

//Phprand
