//Here's an example of several Pbinds being played together to emulate the introduction to "Such Great Heights" by the Postal Service.

/*
http://www.youtube.com/watch?v=0wrsZog8qXg
*/

//First, boot the server add a simple SynthDef.
(
Server.local.waitForBoot({
	SynthDef(\sineTone, {
		arg	freq=440, nharms=2,
			useFilt=0, cf=2000,
			atk=0.001, sus=0.08, rel=0.001,
			peak=1, decLev=0.01,
			amp=0.75, pan=0,
			outBus=0;
		var env, sig;
		env = EnvGen.kr(Env.new([0,peak,decLev,0],[atk,sus,rel],[1,-1,-1]),doneAction:2);
		sig = Blip.ar(freq, nharms);
		sig = Select.ar(useFilt, [sig, LPF.ar(sig, cf)]);
		sig = sig * env;
		sig = Pan2.ar(sig, pan, amp);
		sig = Out.ar(outBus, sig);
	}).add;
})
)

Synth(\sineTone); //make sure it works

//Define several Pbinds. Bear in mind that if an argument is not specified, the default value will be used.
(
~melody1 = Pbind(
	\type,		\note,
	\instrument,\sineTone,
	\dur,		0.17,
	\midinote,	Pseq([
						65,72,65,65,72,65,65,72,
						65,72,65,65,72,65,72,65,
						64,72,64,64,72,64,64,72,
						64,72,64,64,72,64,64,72,
						], inf),
	\pan,		Prand([-0.85,0.85],inf),
	\decLev,	0.2,
	\amp,		0.3,
	\sus,		0.08,
	\outBus,	0
);

~melody2 = Pbind(
	\type,		\note,
	\instrument,\sineTone,
	\dur,		Pseq([
					Pseq([0.34],8),
					Pseq([0.34],8),
					Pseq([0.34],7), Pseq([0.17],2),
					Pseq([0.34],8)
					], inf),
	\midinote,	Pseq([
					Pseq([72],4),
					Pseq([76],4),
					Pseq([72],4),
					Pseq([76],4),
					Pseq([74],4),
					Pseq([74,74,74,76,79],1),
					Pseq([72],4),
					Pseq([76],4)
					], inf),
	\nharms,		1,
	\amp,		0.15,
	\atk,		0.01,
	\sus,		0.25,
	\decLev,	0.5,
	\outBus,	0
);

~bass = Pbind(
	\type,			\note,
	\instrument,	\sineTone,
	\dur,			0.34,
	\midinote,		Pseq([
						Pseq([29],8),
						Pseq([33],8),
						Pseq([31],8),
						Pseq([29],8),
						Pseq([33],8),
						Pseq([31],8)
					],inf),
	\nharms,		8,
	\useFilt,		1,
	\decLev,		0.8,
	\atk,			0.02,
	\sus,			0.33,
	\rel,			0.02,
	\amp,			0.36,
	\outBus,		0
);
)

//Ppar is a pattern that will play several Event streams in parallel. Here we'll synchronize ~melody1 with ~melody2:
x = Ppar([~melody1,~melody2],inf).play;

//We can add the bass pattern, but it will start independently.
y = ~bass.play;


//If we wanted to synchronize all three patterns, we could put all three in a Ppar and play it. But suppose we want the bass pattern to enter later on. EventStreamPlayers can be quantized to a specified delta time. Our smallest subdivision is 0.17 seconds, which can easily be heard as a 16th note. So, we'll quantize to 16 times 0.17, which represents the downbeat of a measure in a 4/4 time signature. Play the first pattern, and execute the next pattern whenever you feel like it-- it'll start playing on the next downbeat:

x = Ppar([~melody1,~melody2],inf).play(quant:0.17*16);

y = ~bass.play(quant:0.17*16);

//See the Pbind help file for more information.

(
x.stop;
y.stop;
)