//Using MIDI to Modify Patterns in Real-Time

//Here's a SynthDef from a previous lecture:
(
SynthDef(\filtSaw, {
	arg		freq=440, detune=2.0, atk=6, sus=0.1, rel=6,
			minCf=30, maxCf=6000, minRq=0.005, maxRq=0.04,
			lowShelf=220, rs=0.85, db=6,
			amp=0.5, out=0;
	var sig, env;
	env = EnvGen.kr(
		Env.new(
			[0,1,1,0],
			[atk,sus,rel],
			[1,0,-1]
		),
		doneAction:2
	);
	sig = Saw.ar(
		freq +
		LFNoise1.kr({LFNoise1.kr(0.5).range(0.15,0.4)}!8).range(detune.neg,detune));
	sig = BPF.ar(
		sig,
		LFNoise1.kr({LFNoise1.kr(0.13).exprange(0.02,0.25)}!8).exprange(minCf, maxCf),
		LFNoise1.kr({LFNoise1.kr(0.08).exprange(0.08,0.35)}!8).range(minRq, maxRq)
	);
	sig = BLowShelf.ar(sig, lowShelf, rs, db);
	sig = Splay.ar(sig);
	sig = sig * env * 10 * amp;
	sig = Limiter.ar(sig, 0.95);
	Out.ar(out, sig);
}).add;
)

//test it out
Synth(\filtSaw);

//Here's a Pbind that will algorithmically generate \filtSaw Synths. As you can see, numerous value patterns have been wrapped in Pdefn patterns, and can therefore be re-defined as the EventStreamPlayer is playing.
(
p = Pbind(
	\instrument, \filtSaw,
	\dur, Pdefn(\sawDur, Pwhite(0.1,1,inf)),
	\midinote, Pdefn(\sawFreq, Pwhite(30,80,inf)),
	\atk, Pdefn(\sawAtk, Plprand(0.01,0.2,inf)),
	\rel, Pdefn(\sawRel, Pwhite(0.05,0.5,inf)),
	\amp, Pdefn(\sawAmp, 0.35),
);
)

//here's what it sounds like by default:
x = p.play;

//let's stop it for now,
x.stop

//and fire up some MIDI
MIDIIn.connectAll;

//Here are two MIDIdefs that will respond to certain CC messages and note-on messages.
(
//You might need to change num==18 to a different value if you want to work with your own hardware.
MIDIdef.cc(\cc1, {
	arg val, num, chan, src;

	//in this case, if we receive a CC message on CC#18,
	if(num == 18, {

		//redefine the attack and release patterns to vary between 0.01 seconds and 1 second. When the CC value is 0, the attack and release time will be 0.01. When the CC value is 127, the attack and release time will be 1.
		Pdefn(\sawAtk, Pseq([val.linexp(0,127,0.01,1)],inf)).trace;
		Pdefn(\sawRel, Pseq([val.linexp(0,127,0.01,1)],inf)).trace;

		//post the received values, because why not?
		[val, num, chan, src].postln;
	});
});

//Similarly, this note-on MIDIdef will limit the freqency
MIDIdef.noteOn(\note1, {
	arg vel, num, chan, src;
	[vel, num, chan, src].postln;


	//uncommented by default, this line will produce random octaves of the received pitch.
	Pdefn(\sawFreq, Prand((num%12) + (24,36..84), inf));


	//uncomment this line to produce pitches within a whole step of the received pitch.
	//change 2 to 2.0 to introduce microtones.

	//Pdefn(\sawFreq, Pwhite(num-2.0,num+2,inf)).trace;

	//the amplitude pattern is controlled by the note velocity.
	Pdefn(\sawAmp, Pseq([vel.linexp(1,127,0.05,0.35)],inf));
});
)

//After creating these MIDIdefs, play the Pbind again:

x = p.play;

x.stop;
