s.reboot.makeGui;

//Sound files for demonstration - you'll need to change the path names on your own computer
(
~jmhcar = Buffer.read(s, "/Users/eli/Desktop/john_car_ignition_01_mono.aiff");
~sn3 = Buffer.read(s, "/Users/eli/Desktop/street_noise_excerpt.aiff");
~hns1m = Buffer.read(s, Platform.resourceDir++"/sounds/car_piece/car_horn_short_1_mono.aiff");
~pt1m = Buffer.readChannel(s, "/Applications/SuperCollider3.6noIDE/SuperCollider.app/Contents/Resources/sounds/piano_trail_01_m.aiff", channels:[0]);
)



//GrainSin.ar

//as simple as it gets
(
x = {
	var sig;
	sig = GrainSin.ar(
		1,				//number of channels
		Impulse.kr(10),	//grain trigger
		0.03,			//grain duration (sec)
		440				//sine frequency
	);
	sig = sig * 0.2;
}.play
)

x.free;

//asynchronous granular synthesis
(
x = {
	var sig;
	sig = GrainSin.ar(
		1,			//number of channels
		Dust.kr(10),	//grain trigger
		0.03,		//grain duration (sec)
		440			//sine frequency
	);
	sig = sig * 0.2;
}.play
)

x.free;

//stereo version
//grain length, pitch, and pan are randomized with LFNoise1
(
x = {
	var sig;
	sig = GrainSin.ar(
		2,
		Dust.kr(10!2),
		LFNoise1.kr.exprange(0.05,0.1),
		LFNoise1.kr.exprange(200,1000),
		LFNoise1.kr.range(-0.5,0.5)
	);
	sig = sig * 0.2;
}.play
)

x.free;

//very short grains
(
x = {
	var sig;
	sig = GrainSin.ar(
		2,
		Dust.kr(10!2),
		LFNoise1.kr.exprange(0.002,0.01),
		LFNoise1.kr.exprange(200,1000),
		LFNoise1.kr.range(-0.5,0.5)
	);
	sig = sig * 0.2;
}.play
)

x.free;



//GrainIn.ar

//expects an input signal instead of a frequency value

//in this case, identical to GrainSin
(
x = {
	var sig;
	sig = GrainIn.ar(
		1,
		Dust.kr(10),
		0.03,
		SinOsc.ar(440)
	);
	sig = sig * 0.2;
}.play
)

x.free;

//USE HEADPHONES TO AVOID FEEDBACK!
//granulating a microphone signal
(
x = {
	var sig;
	sig = GrainIn.ar(
		1,
		Dust.kr(10),
		0.03,
		SoundIn.ar(0)
	);
	sig = sig * 0.2;
}.play
)

x.free;

//USE HEADPHONES TO AVOID FEEDBACK!
//GrainIn, slightly improved via delay lines
//(cheap version of true live granulation)
(
x = {
	var sig;
	sig = SoundIn.ar(0);
	sig = DelayN.ar(sig, 0.2, 0.2);
	sig = sig + AllpassN.ar(sig, 0.62, 0.62, 2.7);
	sig = sig + AllpassN.ar(sig, 0.35, 0.35, 1.6);
	sig = GrainIn.ar(

		//number of channels
		2,

		//grain trigger
		Dust.kr(3.5),

		//grain duration
		LFNoise1.kr(5).range(0.05,0.13),

		//input signal
		sig,

		//pan position
		LFNoise1.kr(10)
	);
}.play
)

x.free;

//GrainBuf - the good stuff
//Remember that GrainBuf must be provided with a MONOPHONIC buffer!

//simple stereo version
(
x = {
	var sig;
	sig = GrainBuf.ar(
		2,				//num channels
		Dust.kr(8!2),		//grain trigger
		0.03,			//grain length (sec)
		~jmhcar.bufnum,	//buffer to granulate
		1,				//playback rate
		0.5,				//grain pointer position (0-1)
		4,				//interpolation
		0				//pan position
	);
	sig = sig * 0.2;
}.play
)

x.free;

//randomize grain pointer position
(
x = {
	var sig;
	sig = GrainBuf.ar(
		2,
		Dust.kr(8!2),
		0.03,
		~jmhcar.bufnum,
		1,
		LFNoise1.kr.range(0,0.99),
		4,
		0
	);
	sig = sig * 0.2;
}.play
)

x.free;

//also randomize grain duration
(
x = {
	var sig;
	sig = GrainBuf.ar(
		2,
		Dust.kr(8!2),
		LFNoise1.kr.exprange(0.003,0.12),
		~jmhcar.bufnum,
		1,
		LFNoise1.kr.range(0,0.99),
		4,
		0
	);
	sig = sig * 0.2;
}.play
)

x.free;

//also randomize grain playback rate
(
x = {
	var sig;
	sig = GrainBuf.ar(
		2,
		Dust.kr(8!2),
		LFNoise1.kr.exprange(0.003,0.12),
		~jmhcar.bufnum,
		LFNoise1.kr.exprange(0.5,2),
		LFNoise1.kr.range(0,0.99),
		4,
		0
	);
	sig = sig * 0.2;
}.play
)

x.free;

//In class I neglected to show you how to create and use a custom grain envelope. So far, we've only been using the default Hann window. Here's how you can make your own grain envelope shape (this can also be found in the examples within the GrainBuf helpfile):

//create an Env (Env is a language object, and can be used independently of the EnvGen UGen). Keep in mind that you can use any non-sustaining Env method (Env.new, Env.sine, Env.triangle, Env.perc, etc). This Env.new has a sharp attack time:
~myEnv = Env.new([0,1,0],[0.001,0.1],[5,-1]);

//Discretize the Env (convert the abstract shape into discrete points), load the values into a Buffer, and send it to the server:
~myEnvBuf = Buffer.sendCollection(s, ~myEnv.discretize);

//We can now use our grain envelope in a GrainBuf (or any granular UGen) by providing our Buffer's bufnum for GrainBuf's envbufnum argument:
(
x = {
	var sig;
	sig = GrainBuf.ar(
		2,
		Dust.kr(4!2),
		LFNoise1.kr.exprange(0.05,0.12),
		~jmhcar.bufnum,
		LFNoise1.kr.exprange(0.5,2),
		LFNoise1.kr.range(0,0.99),
		4,
		0,
		~myEnvBuf.bufnum
	);
	sig = sig * 0.2;
}.play
)

x.free;

//Notice the auditory difference between our custom grain envelope and the built-in Hann envelope, which has a smoother attack:
(
x = {
	var sig;
	sig = GrainBuf.ar(
		2,
		Dust.kr(4!2),
		LFNoise1.kr.exprange(0.05,0.12),
		~jmhcar.bufnum,
		LFNoise1.kr.exprange(0.5,2),
		LFNoise1.kr.range(0,0.99),
		4,
		0,
		-1
	);
	sig = sig * 0.2;
}.play
)

x.free;

//Here's a third envelope with a reverse percussive shape:
(
~myEnv2 = Env.new([0,1,0],[0.1,0.001],[3,-1]);
~myEnv2Buf = Buffer.sendCollection(s, ~myEnv2.discretize);
)

(
x = {
	var sig;
	sig = GrainBuf.ar(
		2,
		Dust.kr(4!2),
		LFNoise1.kr.exprange(0.05,0.12),
		~jmhcar.bufnum,
		LFNoise1.kr.exprange(0.5,2),
		LFNoise1.kr.range(0,0.99),
		4,
		0,
		~myEnv2Buf
	);
	sig = sig * 0.2;
}.play
)

x.free;





//examples from fixed media pieces

/*
from Short Ride in a Used '98 Honda, Mvt I
http://www.elifieldsteel.com/works/fixed-media/short-ride-in-a-used-98-honda
*/

(
{
	var sig, pchEnv;

	//controlling grain playback rate with an EnvGen
	pchEnv = EnvGen.kr(Env.new([0.2,0.2,0.6],[5,10],\exponential));

	sig = GrainBuf.ar(

		//number of channels
		2,

		//grain trigger
		Impulse.ar(XLine.kr(8,400,15,doneAction:2)),
		//grain duration
		Line.kr(0.15,0.04,15),

		//buffer
		~sn3.bufnum,

		//playback rate
		pchEnv,

		//grain pointer position
		Line.ar(0,0.7,15),

		//interpolation
		4,

		//pan position
		LFNoise1.kr(0.3).range(-0.5,0.5),

		mul:2.2
	);

}.play
)

(
{
	var sig, pchEnv;

	pchEnv = EnvGen.kr(Env.new([1,1,0.4],[13,7],\exponential));

	sig = GrainBuf.ar(

		//number of channels
		2,

		//grain trigger
		Impulse.ar(XLine.kr(2,250,20,doneAction:2)),

		//grain duration
		Line.kr(0.15,0.04,20),

		//buffer
		~sn3.bufnum,

		//playback rate
		pchEnv,

		//grain pointer position
		Line.ar(0,1,20),

		//interpolation
		4,

		//pan position
		LFNoise1.kr(3).range(-0.5,0.5),

		//amplification
		mul:3
	);

}.play
)


/*
from Short Ride in a Used '98 Honda, Mvt II
http://www.elifieldsteel.com/works/fixed-media/short-ride-in-a-used-98-honda
*/

//TGrains is a lot like GrainBuf, with only a few minor differences, in particular the use of a centerPos argument for grain generation (in seconds), rather than a startPos argument for grain generation, ranging from 0 to 1.
(
{
	var sig;
	sig = TGrains.ar(

		//number of channels
		2,

		//grain trigger
		Impulse.kr(XLine.kr(0.8,18,3,doneAction:2)),

		//buffer
		~hns1m.bufnum,

		//playback rate
		XLine.kr(1,0.85,3),

		//center position (seconds)
		Line.kr(0.2, 0.01, 3),

		//grain duration (seconds)
		LFNoise1.kr(15).range(0.03,0.19),

		//pan position
		LFNoise1.kr(14).range(-0.5,0.5),

		//amplitude
		XLine.kr(0.16,0.8,3)
		);

}.play
)

(
{
	var sig, ampEnv, cPosEnv, rateEnv, densityEnv;

	ampEnv = EnvGen.kr(Env([0.2,0.8,0.3],[5,5],\lin),doneAction:2);
	cPosEnv = EnvGen.kr(Env([0.01,0.14,0.01],[5,5],\lin));
	rateEnv = EnvGen.kr(Env([0.75,0.75,0.95],[5,5],\lin));
	densityEnv = EnvGen.kr(Env([2,20,4],[5,5],\exponential));

	sig = TGrains.ar(

		//number of channels
		2,

		//grain trigger
		Dust.kr(densityEnv),

		//buffer
		~hns1m.bufnum,

		//playback rate
		rateEnv,

		//center position (seconds)
		cPosEnv,

		//grain duration (seconds)
		LFNoise1.kr(25).range(0.02,0.08),

		//pan position
		SinOsc.kr(LFNoise1.kr(3).range(0.2,3.0)).range(-0.8,0.8),

		//amplitude
		ampEnv,
		);

}.play
)

(
{
	var sig;

	sig = TGrains.ar(

		//number of channels
		2,

		//grain trigger
		Select.kr(
			TRand.kr(0,1,Impulse.kr(10)) > 0.85,
			Impulse.kr([16,20])
		),

		//buffer
		~pt1m.bufnum,

		//playback rate
		1,

		//center position (seconds)
		0.3,

		//grai duration (seconds)
		LFNoise2.kr([2.3,3.35]).range(0.03, 0.25),

		//pan position
		[
			LFNoise2.kr(0.2, mul:0.75),
			LFNoise2.kr(0.25, mul:0.75)
		],

		//amplitude
		BrownNoise.kr.range(0.1,0.9);
	);

}.play
)