s.boot;

//use whatever sound file you'd like to granulate
b = Buffer.read(s, "/Users/eli/Desktop/piano_trail_01_m.aiff");

(
SynthDef(\sync_async, {
	arg type=0, gate=0, lagTime=0.1,
	grainHz=8, syncProb=1, buf, rate=1, gdur=0.03,
	pan=1, amp=1, out=0;
	var xfade, asyncTrig, syncTrig, trig, panPos, sig;

	//asynchronous granular synthesis: type == 0
	//synchronous granular synthesis: type == 1

	//don't allow ridiculous values
	type = type.clip(0,1);

	//xfade = EnvGen.kr(Env.new([1,1],[0],\lin), t_fade, levelScale:type);
	xfade = VarLag.kr(type, lagTime, 0).poll(label:"xfade");

	//grain triggers for asynchronous/synchronous synthesis
	asyncTrig = Dust.ar(grainHz);
	syncTrig = Impulse.ar(grainHz);

	//CoinGate uses a probability (between 0 and 1) to determine whether to pass a trigger
	asyncTrig = CoinGate.ar(1-xfade, asyncTrig);
	syncTrig = CoinGate.ar(xfade, syncTrig);

	//when dealing with synchronous granular synthesis, it's nice to be able to randomly "drop" grains and create an interesting, irregular rhythm. This additional CoinGate is applied to the synchronous triggers in order to filter out some of them
	syncTrig = CoinGate.ar(syncProb, syncTrig);

	//the sum of these two trigger signals will determine when grains are produced
	trig = asyncTrig + syncTrig;

	//don't allow stupid pan values
	pan = pan.clip(0,1);

	//pan position spread
	panPos = LFNoise1.kr.bipolar(pan);

	//synthesize grains
	sig = GrainBuf.ar(2, trig, gdur, buf, rate, LFNoise1.kr.range(0,0.9), 2, panPos);

	//amplitude scaling
	sig = sig * amp;

	Out.ar(out, sig);
}).add;
)


//inidividual synth instance with default values
x = Synth(\sync_async);

//over 20 seconds, fade from asynchronous to synchronous
x.set(\lagTime, 20, \type, 1);

//Lag UGens can be interrupted. We'll fade from synchronous to asynchronous over 30 seconds, but before the fade is complete, run the next line, which fades back to synchronous over 3 seconds.

x.set(\lagTime, 30, \type, 0);
x.set(\lagTime, 3, \type, 1);

//now that we're in synchronous mode, we can modify the value of syncProb in order to randomly "drop" synchronous grains.

//50% grain probability
x.set(\syncProb, 0.5);

//20% chance
x.set(\syncProb, 0.2);

//80% chance
x.set(\syncProb, 0.8);

//grain frequency can be changed
x.set(\grainHz, 12);

//and grain duration as well
x.set(\gdur, 0.01);

//free the synth
x.free;


//Using s.bind, we can create three instances of this synth that are instantiated at the exact same time. I'm also placing all three synths in a group called g. Each has a unique transposition value.
(
s.bind({
	g = Group.new;
	x = Synth(\sync_async, [\rate, 1], g);
	y = Synth(\sync_async, [\rate, -7.midiratio], g);
	z = Synth(\sync_async, [\rate, 7.midiratio], g);
});
)

//At first, all three are asynchronous, but we can fade them all to synchronous mode by sending a single message to the group they belong to:
g.set(\lagTime, 20, \type, 1);

//Type doesn't have to be conceived as a boolean value. We can set the type to be 0.95 in order to have "mostly synchronous" granular synthesis. We'll fade to this decimal type value over 5 seconds.
g.set(\lagTime, 5, \type, 0.95);

//Suppose we want to drop some of these grains. We can send another message to the group in order to modify the value of syncProb for each of the three synths.
g.set(\syncProb, 0.3);

//Notice that because each synth has its own instance of CoinGate, the decision of which grains to drop is unique to each synth.

//free the group and its synths
g.free