//Routing MIDI between SuperCollider and Other MIDI-Aware Applications

//Mac OS X includes a useful application called IAC (Inter-Application Communication), which allows one to create virtual MIDI ports which can be used by MIDI-aware applications to send and receive MIDI data.

/*
1. In Applications >> Utilities >> Audio MIDI setup, view the MIDI Studio window if it is not already visible by clicking Show MIDI Window in the Window drop-down menu.

2. Double-click the IAC Driver icon to open the IAC dialog.

3. Click the "+" icon in the lower left corner to create a new virtual MIDI port. You may want to double-click and rename the new port to something more meaningful.

4. Make sure the box labeled "Device is online" is checked.
*/

//In SuperCollider (which you may have to relaunch), initialize MIDI communication by running the following line of code:

MIDIClient.init;

//SuperCollider will post information about available MIDI sources and destinations. View an Array of MIDI destinations by running the following line of code:

MIDIClient.destinations;

//Your newly created port should appear in this list, and it is most likely the last item. SC may truncate the code dumped to the post window, so here's one way you can interatively list all of the items in the Array of MIDI destinations:

MIDIClient.destinations.do({arg item; item.postln});nil;

//To send MIDI messages from SuperCollider (so far we've only been receiving them), we need to make an instance of MIDIOut. MIDIOut needs to know what port to send messages to. More specifically, it needs the index of that destination's location in the MIDIClient.destinations Array. It's mostly likely the last item in the Array, and if so, here's a quick way of getting that index -- it's simply the size of the Array minus 1.

m = MIDIOut.new(MIDIClient.destinations.size-1);

//We can now use instance methods such as "noteOn", "noteOff", "control", etc (see the MIDIOut help file for more options), and we can generate these messages algorithmically, manually, or otherwise.

//Like SuperCollider, any other MIDI-aware application (such as a DAW, MaxMSP, etc) can send or receive MIDI message on your newly created virtual MIDI port. Open up such an application and make sure it is listening to the appropriate port and channel(s).

//Remember that most applications understand MIDI channels as ranging from 1 to 16, but SuperCollider understands them as 0 to 15. The following code establishes a Routine (remember that "fork" turns a Function into a Routine) that generates noteOns and noteOffs. It randomly chooses a channel between 0 and 3, as well as a random note number and velocity. It creates a noteOn message, waits for a random period of time, then generates a noteOff message.

//Make sure your other application is listening to these channels for noteOns and noteOffs. When you evaluate the following code, you should receive the data in the other application.

(
~makeNotes = {
	inf.do{
		var chan, nn, vel;
		chan = rrand(0,3);
		nn = rrand(40,90);
		vel = rrand(1,127);
		m.noteOn(chan, nn, vel);
		exprand(0.01,1.5).wait;
		m.noteOff(chan,nn);
	};
}.fork;
)

~makeNotes.stop;

//Using this technique of sending MIDI messages to a virtual port, the algorithmic strengths of SuperCollider can be used to "drive" other applications, sample libraries, or VST plugins (Vienna, EastWest, Absynth, FM8, etc etc etc...)