//MIDI Events with Pbind

//As mentioned in previous lectures, Pbind generates a stream of Events by binding values to keys. The \note Event is the default event type and is the most common, but there are many types of Events. See the following help file for more information on Event types and the keys that are inherent to those types of Events:

/*
PG_08_Event_Types_and_Parameters
*/

//In general, I strongly advise that you read the Practical Guide to Patterns, starting with this help file (It's considerably more helpful and more well-written than most of the SC help documentation):

/*
PG_01_Introduction
*/

//Anyway, we can use Pbind to generate MIDI Events, thereby harnessing the power of Patterns and using it to drive other MIDI-aware applications. See the previous document for information on setting up a virtual port using IAC.

//Inititalize SC's MIDI awareness
MIDIClient.init;

//Make sure your virtual port is available in the list of destinations
MIDIClient.destinations[0];

//and create an instance of MIDIOut using the index of that virtual port:
m = MIDIOut.new(0);

//Test to see if MIDI communication is working correctly:
m.noteOn(0, 60, 64);
m.noteOff(0, 60);

(
x = Pbind(
	\type, \midi,
	\midicmd, \noteOn,
	\midiout, m,
	\chan, 0,
	\dur, 0.09,
	\midinote, Pdefn(\nn, Pxrand([28,40,50,52,55,56,64,67,68],inf)),
	\ctranspose, Pdefn(\trnsp, 0),
	\amp, Pdefn(\ampl, Plprand(0.01,1,inf)),
	\sustain, Pdefn(\sus, Plprand(0.02,0.1,inf)),
);
)

z = x.play;

//change transposition
Pdefn(\trnsp, 0);

//A new random Array of note numbers
Pdefn(\nn, Pseq(Array.fill(rrand(20,25),{rrand(40,100)}).postln,inf));

//sorted
Pdefn(\nn, Pseq(Array.fill(rrand(20,25),{rrand(40,100)}).sort.postln,inf));

//major triads
Pdefn(\nn, Pseq((Array.fill(rrand(20,25),{rrand(40,100)}).round(12).collect({|n| n+([0,4,7].choose)})-rrand(0,12)).sort.postln,inf));

//gradual recurring 10-second crescendo
Pdefn(\ampl, Pseg(Pseq([0.2,1],inf), Pseq([10,0.1],inf), \exponential));

//longer notes
Pdefn(\sus, Plprand(0.5,1.5,inf));

z.stop;